<?php

namespace Util;

use Zeedhi\Framework\Session\Session;
use Teknisa\Libs\Util\Environment as LibEnvironment;
use ZeedhiUser\Services\AccessControl\Environment as ZhuEnvironment;

class Environment extends LibEnvironment {

    const TOKEN = "token";

    const USR_ID = "userId";
    const USR_NAME = "username";
    const USR_EMAIL = "userEmail";
    const ORG_ID = "organizationId";
    const ORG_NAME = "organizationName";
    const CLIENT_NRORG = "clientNrOrg";
    const CLIENT_NRORG_PRINCIPAL = "clientNrOrgPrincipal";
    const NRORGOPER = "NRORGOPER";
    const ACL_PROFILE = "accessProfiles";
    const ACL_PROFILE_LOGIN = 'ACCESS_PROFILE';
    const URL = "url";
    const USR_IMG = "userImg";
    const USER_ADMIN = "userAdmin";
    const MIGRATION = "migration";

    /** @var Session $session */
    protected $session;
    /** @var \Zmart\Mongo\Mongo  $mongo */
    protected $mongo;

    public function __construct(Session $session, $mongo) {
        $this->session = $session;
        $this->mongo = $mongo;
        $this->session->start();
        $nrorg = $this->getOrganizationId();
        if($nrorg){
            $this->setDBOrganization($this->getOrganizationId());
        }
    }

    public function getToken() {
        return $this->session->get(self::TOKEN);
    }

    public function getUserId() {
        return $this->getCurrentUserId();
    }

    public function getUserName() {
        return $this->getCurrentUserName();
    }

    public function getUserEmail() {
        return $this->getCurrentEmail();
    }

    public function getOrganizationId() {
        return $this->getCurrentOrganizationId();
    }

    public function getOrganizationName() {
        return $this->getCurrentTradingName();
    }

    public function getAclProfiles() {
        return $this->session->get(self::ACL_PROFILE);
    }
    public function getAclProfilesLogin() {
        return $this->session->get(self::ACL_PROFILE_LOGIN);
    }

    public function getUserUrl() {
        return $this->session->get(self::URL);
    }

    public function setToken($token) {
        $this->session->set(self::TOKEN, $token);
    }

    public function setUserId($userId) {
        $this->session->set(self::USR_ID, $userId);
    }

    public function setUserName($userName) {
        $this->session->set(self::USR_NAME, $userName);
    }

    public function setUserEmail($userEmail) {
        $this->session->set(self::USR_EMAIL, $userEmail);
    }

    public function setOrganizationId($organizationId) {
        $this->session->set(self::ORG_ID, $organizationId);
        $this->setDBOrganization($organizationId);
    }

    public function setCurrentTradingName($name) {
        $this->session->set("TRADING_NAME",$name);
    }

    protected function setDBOrganization($organizationId) {
        $currentNrOrg = self::getNrOrgClient();
        $nrorgoper = self::getNrorgoper();
        if($nrorgoper){
            $organizationId = $nrorgoper;
        }
        if($currentNrOrg){
            $organizationId = $currentNrOrg;
        }
        $this->mongo->setDbByName($organizationId);
    }

    public function setNrOrgClient($clientNrOrg) {
        $this->session->set(self::CLIENT_NRORG, $clientNrOrg);
    }

    public function getNrOrgClient() {
       return $this->session->get(self::CLIENT_NRORG);
    }

    public function setNrOrgClientPrincipal($clientNrOrgPrincipal) {
        $this->session->set(self::CLIENT_NRORG_PRINCIPAL, $clientNrOrgPrincipal);
    }

    public function getNrOrgClientPrincipal() {
       return $this->session->has(self::CLIENT_NRORG_PRINCIPAL) ?
           $this->session->get(self::CLIENT_NRORG_PRINCIPAL) :
           null;

    }

    public function setOrganizationName($organizationName) {
        $this->session->set(self::ORG_NAME, $organizationName);
    }

    public function setNrorgoper($nrorgoper) {
        $this->session->set(self::NRORGOPER, $nrorgoper);
    }

    public function getNrorgoper() {
        return $this->session->get(self::NRORGOPER);
    }

    public function setAclProfiles($aclProfiles) {
        $this->session->set(self::ACL_PROFILE, $aclProfiles);
    }

    public function setUserUrl($userUrl) {
        $this->session->get(self::URL, $userUrl);
    }

    public function destroySession() {
        $this->session->destroy();
    }

    public function cleanSession() {
        $this->session->clear();
    }

    public function getCurrentTime() {
        return new \DateTime();
    }

    public function getProjectId() {
        return 3;
    }

    public function getProjectUniqueId() {
        return "Zmart_3";
    }

    public function getContainersJsonPath() {
        return realpath("../../mobile/assets/json/containers.json");
    }

    public function getJsonsDirectories() {
        return array(realpath("../../mobile/assets/json/containers"));
    }

    public function getDataBaseMapFilePath() {
        return realpath("../src/Json/mongoMap.json");
    }
}
