import {ChartViewService} from "@/Services/ChartViewService";
import {Metadata} from "@zeedhi/core";
import {Col, Form, ISelect, IFormComponent, Button} from "@zeedhi/common";
import { ChartViewController } from '../../controllers/ChartViewController';
import { Utilities } from '../Utilities';

export class ZmartChartsApex {
    public charts:any;
    public chartMetadata:any = {};
    public chartContext:any = {};

    public tela:any = {};
    public drillDownFilters:any = {};
    public chartViewController: any;
    private factMaskFunction: any;
    private dimensionMaskFunction: any;

    public static build(chartMetadata:any, chartViewController:any, component:any) {
        const instacia = new this();
        instacia.setChartMetadata(chartMetadata);
        if (chartMetadata[0].name !== 'notFound') {
            instacia.setChartViewController(chartViewController);
            instacia.create(chartMetadata);
        } else {
            instacia.charts = chartMetadata;
        }
        instacia.setTela(component);
        return instacia;
    }

    public getCharts() {
        return this.charts;
    }

    public setTela(tela:any) {
        this.tela = tela;
    }

    public async drill(event: any, chartContext: any, config: any) {
        if (config.dataPointIndex > -1) {
            const fieldDrill = config.config.xaxis.field;
            const valueDrill = config.config.xaxis.categories[config.dataPointIndex];
            const drillDownView = config.config.drillDownView;
            let drillDownFilters:any = {};
            if (this.drillDownFilters[config.globals.chartID]) {
                drillDownFilters = this.drillDownFilters[config.globals.chartID];
            }
            drillDownFilters[fieldDrill] = valueDrill;
            await this.chartViewController.buildWidgetByViewId(this.tela, drillDownView, drillDownFilters);
        }
    }

    public getFactMaskFunction() {
        return this.factMaskFunction;
    }

    public factFormatter(value: any, apex: any) {
        let valueMask = value;
        if (!isNaN(value) && value != null) {
            valueMask = this.factMaskFunction(value);
        }
        return valueMask;
    }

    public dimensionFormatter(value: any, apex: any) {
        let valueMask = value;
        valueMask = this.dimensionMaskFunction(value);
        return valueMask;
    }

    public dataLabelsFormatter(value: any, apex: any) {
        if (apex.w.config.chart.formatValue) {
            if (apex.w.config.dataLabels.showAsPercent) {
                value = Utilities.roundFloat(value);
            } else {
                value = apex.w.config.series[apex.seriesIndex];
            }
        }
        let valueMask = value;
        if (!isNaN(value)) {
            if (apex.w.config.chart.stacked) {
                if (apex.seriesIndex === ((apex.w.config.series.length) - 1)) {
                    valueMask = this.factMaskFunction(apex.w.globals.stackedSeriesTotals[apex.dataPointIndex]);
                } else {
                    valueMask = "";
                }
            } else if (value != null) {
                valueMask = this.factMaskFunction(value);
            }
        }
        return valueMask;
    }

    public setChartContext(chartContext:any) {
        this.chartContext = chartContext;
    }

    public mounted(chartContext:any, config:any) {
        this.setChartContext(chartContext);
    }

    private setCardFormatterFunctions(card: any) {
        const cardValueElement = card[0].children[1];
        const cardValue = cardValueElement.text.replace(/<\/?[^>]+(>|$)/g, '');
        const maskedValue = ChartViewService.getMaskFunction(card[0].widget.cardMask)(Number(cardValue));
        cardValueElement.text = cardValueElement.text.replace(cardValue, maskedValue);
    }

    private setEChartsFormatterFunctions(chart: any) {
        chart[0].options.series[0].axisLabel.formatter = function (value: any) {
            return Utilities.roundFloat(value, 2);
        };
    }

    private setFormatterFunctions(charts: any) {
        let factAxis = "yaxis";
        let dimensionAxis = "xaxis";
        if (charts[0].invertedAxis) {
            factAxis = "xaxis";
            dimensionAxis = "yaxis";
        }
        this.dimensionMaskFunction = ChartViewService.getMaskFunction(charts[0].options.xaxis.mask);
        charts[0].options[dimensionAxis].labels.formatter = this.dimensionFormatter.bind(this);

        if (charts[0].options[factAxis].length) {
            for (let i = 0; i < charts[0].options[factAxis].length; i += 1) {
                this.factMaskFunction = ChartViewService.getMaskFunction(charts[0].options.yaxis[i].mask);
                charts[0].options[factAxis][i].labels.formatter = this.factFormatter.bind(this);
            }
            charts[0].options.dataLabels.formatter = ChartViewService.getMaskFunctionDynamic;
        } else {
            this.factMaskFunction = ChartViewService.getMaskFunction(charts[0].options.yaxis.mask);
            charts[0].options[factAxis].labels.formatter = this.factFormatter.bind(this);
            charts[0].options.dataLabels.formatter = this.dataLabelsFormatter.bind(this);
        }
    }

    public create(charts: any) {
        if (charts[0].component === 'ZdApexChart') {
            this.setFormatterFunctions(charts);

            charts[0].options.chart.events = {
                "click": this.drill.bind(this),
                "mounted": this.mounted.bind(this)
            };
            // charts[0].options.chart.toolbar.tools.customIcons = [{
            //     icon: '<img src="../img/icons/edit.png" width="14" style="padding-top: 2px; margin-top: 2px;">',
            //     index: "0",
            //     title: 'Alterar view',
            //     class: 'custom-icon',
            //     click: this.onclickEditCharts.bind(this)
            // }];
        } else if (charts[0].component !== 'ZdECharts') {
            this.setCardFormatterFunctions(charts);
        } else {
            this.setEChartsFormatterFunctions(charts);
        }
        this.charts = charts;
    }

    public setChartMetadata(chartMetadata: any) {
        this.chartMetadata = chartMetadata;
    }
    public setChartViewController(chartViewController: any) {
        this.chartViewController = chartViewController;
    }

    public formButtonEditClickApplyView() {
        const formViewEdit = Metadata.getInstance<Form>('form-view-edit');
        const currentRow = formViewEdit.value;
        this.charts[0].options.title.text = currentRow.name;

        const type = currentRow.serieType;
        this.charts[0].options.plotOptions.bar.horizontal = currentRow.chartInvertedAxes;
        this.charts[0].options.dataLabels.offsetX = currentRow.offsetX;
        this.charts[0].options.dataLabels.offsetY = currentRow.offsetY;

        if (this.charts[0].chartType !== type) {
            this.charts[0].chartType = type;
        } else {
            this.tela.childrenInstances[1].updateChart(this.charts[0].options);
        }
        this.tela.children = [];
        this.tela.children = this.charts;
    }

    public formButtonEditClickCloseView() {
        ChartViewController.closeEditView();
        this.chartContext.destroy();
        this.tela.childrenInstances = [];
        this.chartViewController.buildWidgetByViewId(this.tela, this.charts[0].id, {}).then();
    }

    public setEventsButton() {
        const formButtonEdit = Metadata.getInstance<Form>('form-button-edit');
        const buttonApplyView = formButtonEdit.getChildInstance<Button>('applyView');
        const buttonCloseView = formButtonEdit.getChildInstance<Button>('closeView');
        buttonApplyView.events.click = this.formButtonEditClickApplyView.bind(this);
        buttonCloseView.events.click = this.formButtonEditClickCloseView.bind(this);
    }
    public formViewEditChangeType() {
        const formViewEdit = Metadata.getInstance<Form>('form-view-edit');
        const currentRow = formViewEdit.value;
        const fieldType = formViewEdit.getChildInstance<ISelect>('type');
        if (fieldType.datasource && fieldType.datasource.currentRow) {
            const fieldTypeCurrentRow = <any>fieldType.datasource.currentRow;
            currentRow.serieType = fieldTypeCurrentRow.serieType ? fieldTypeCurrentRow.serieType : fieldTypeCurrentRow.name;
            currentRow.chartInvertedAxes = !!fieldTypeCurrentRow.chartInvertedAxes;
            currentRow.offsetX = fieldTypeCurrentRow.offsetX ? fieldTypeCurrentRow.offsetX : null;
            currentRow.offsetY = fieldTypeCurrentRow.offsetY ? fieldTypeCurrentRow.offsetY : null;
            // ChartViewController.setFieldsEditView(fieldTypeCurrentRow);
        }
    }

    public async onclickEditCharts(chart: any, options: any, e: any) {
        ChartViewController.openEditView();

        this.setEventsButton();

        const formViewEdit = Metadata.getInstance<Form>('form-view-edit');
        const currentRow = formViewEdit.value;

        currentRow.name = this.charts[0].options.title.text;

        const fieldType = formViewEdit.getChildInstance<ISelect>('type');
        if (fieldType.events) {
            fieldType.events.change = this.formViewEditChangeType.bind(this);
        }
        if (fieldType.datasource) {
            await fieldType.datasource.get();
            fieldType.setValue(this.charts[0].viewTypeId);
            this.formViewEditChangeType();
        }
    }
}
